@@ -23,14 +23,14 @@
 #define __PLOT_H_
 
 #include <ncurses.h>
+#include <stdbool.h>
 #include <stddef.h>
 
-void nvtop_line_plot(WINDOW *win, size_t num_data, const double *data,
-                     double min, double max, unsigned num_plots,
-                     const char *legend[]);
+#define PLOT_MAX_LEGEND_SIZE 35
 
-void nvtop_bar_plot(WINDOW *win, size_t num_data, const double *data,
-                    double min, double max);
+void nvtop_line_plot(WINDOW *win, size_t num_data, const double *data,
+                     unsigned num_plots, bool legend_left,
+                     char legend[4][PLOT_MAX_LEGEND_SIZE]);
 
 void draw_rectangle(WINDOW *win, unsigned startX, unsigned startY,
                     unsigned sizeX, unsigned sizeY);
