@@ -1,6 +1,6 @@
 /*
  *
- * Copyright (C) 2017 Maxime Schmitt <maxime.schmitt91@gmail.com>
+ * Copyright (C) 2017-2021 Maxime Schmitt <maxime.schmitt91@gmail.com>
  *
  * This file is part of Nvtop.
  *
@@ -29,7 +29,10 @@
 
 #include <locale.h>
 
+#include "nvtop/extract_gpuinfo.h"
 #include "nvtop/interface.h"
+#include "nvtop/interface_common.h"
+#include "nvtop/interface_options.h"
 #include "nvtop/time.h"
 #include "nvtop/version.h"
 
@@ -50,12 +53,14 @@ static const char helpstring[] =
     "Available options:\n"
     "  -d --delay        : Select the refresh rate (1 == 0.1s)\n"
     "  -v --version      : Print the version and exit\n"
+    "  -c --config-file  : Provide a custom config file location to load/save "
+    "preferences\n"
     "  -s --gpu-select   : Colon separated list of GPU IDs to monitor\n"
     "  -i --gpu-ignore   : Colon separated list of GPU IDs to ignore\n"
     "  -p --no-plot      : Disable bar plot\n"
-    "  -r --reverse-abs  : Reverse abscissa: plot the recent data left and older on the right\n"
+    "  -r --reverse-abs  : Reverse abscissa: plot the recent data left and "
+    "older on the right\n"
     "  -C --no-color     : No colors\n"
-    "  -N --no-cache     : Always query the system for user names and command "
     "line information\n"
     "  -f --freedom-unit : Use fahrenheit\n"
     "  -E --encode-hide  : Set encode/decode auto hide time in seconds "
@@ -68,9 +73,12 @@ static const struct option long_opts[] = {
     {.name = "delay", .has_arg = required_argument, .flag = NULL, .val = 'd'},
     {.name = "version", .has_arg = no_argument, .flag = NULL, .val = 'v'},
     {.name = "help", .has_arg = no_argument, .flag = NULL, .val = 'h'},
+    {.name = "config-file",
+     .has_arg = required_argument,
+     .flag = NULL,
+     .val = 'c'},
     {.name = "no-color", .has_arg = no_argument, .flag = NULL, .val = 'C'},
     {.name = "no-colour", .has_arg = no_argument, .flag = NULL, .val = 'C'},
-    {.name = "no-cache", .has_arg = no_argument, .flag = NULL, .val = 'N'},
     {.name = "freedom-unit", .has_arg = no_argument, .flag = NULL, .val = 'f'},
     {.name = "gpu-select",
      .has_arg = required_argument,
@@ -89,7 +97,7 @@ static const struct option long_opts[] = {
     {0, 0, 0, 0},
 };
 
-static const char opts[] = "hvd:s:i:CNfE:pr";
+static const char opts[] = "hvd:s:i:c:CfE:pr";
 
 static size_t update_mask_value(const char *str, size_t entry_mask,
                                 bool addTo) {
@@ -125,15 +133,17 @@ int main(int argc, char **argv) {
   (void)setlocale(LC_CTYPE, "");
 
   opterr = 0;
-  int refresh_interval = 1000;
+  bool update_interval_option_set = false;
+  int update_interval_option;
   char *selectedGPU = NULL;
   char *ignoredGPU = NULL;
-  bool use_color_if_available = true;
-  bool cache_pid_infos = true;
-  bool use_fahrenheit = false;
-  bool show_plot = true;
-  bool plot_old_to_recent = true;
-  double encode_decode_hide_time = 30.;
+  bool no_color_option = false;
+  bool use_fahrenheit_option = false;
+  bool hide_plot_option = false;
+  bool reverse_plot_direction_option = false;
+  bool encode_decode_timer_option_set = false;
+  double encode_decode_hide_time = -1.;
+  char *custom_config_file_path = NULL;
   while (true) {
     int optchar = getopt_long(argc, argv, opts, long_opts, NULL);
     if (optchar == -1)
@@ -151,7 +161,12 @@ int main(int argc, char **argv) {
         fprintf(stderr, "Error: A negative delay requires a time machine!\n");
         exit(EXIT_FAILURE);
       }
-      refresh_interval = (int)delay_val * 100u;
+      update_interval_option_set = true;
+      update_interval_option = (int)delay_val * 100u;
+      if (update_interval_option > 99900)
+        update_interval_option = 99900;
+      if (update_interval_option < 100)
+        update_interval_option = 100;
     } break;
     case 's':
       selectedGPU = optarg;
@@ -165,14 +180,14 @@ int main(int argc, char **argv) {
     case 'h':
       printf("%s\n%s", versionString, helpstring);
       exit(EXIT_SUCCESS);
-    case 'C':
-      use_color_if_available = false;
+    case 'c':
+      custom_config_file_path = optarg;
       break;
-    case 'N':
-      cache_pid_infos = false;
+    case 'C':
+      no_color_option = true;
       break;
     case 'f':
-      use_fahrenheit = true;
+      use_fahrenheit_option = true;
       break;
     case 'E': {
       if (sscanf(optarg, "%lf", &encode_decode_hide_time) == EOF) {
@@ -180,12 +195,13 @@ int main(int argc, char **argv) {
                 optarg);
         exit(EXIT_FAILURE);
       }
+      encode_decode_timer_option_set = true;
     } break;
     case 'p':
-      show_plot = false;
+      hide_plot_option = true;
       break;
     case 'r':
-       plot_old_to_recent = false;
+      reverse_plot_direction_option = true;
       break;
     case ':':
     case '?':
@@ -224,56 +240,105 @@ int main(int argc, char **argv) {
     exit(EXIT_FAILURE);
   }
 
-  size_t gpu_mask;
+  size_t gpu_mask_nvidia;
   if (selectedGPU != NULL) {
-    gpu_mask = 0;
-    gpu_mask = update_mask_value(selectedGPU, gpu_mask, true);
+    gpu_mask_nvidia = 0;
+    gpu_mask_nvidia = update_mask_value(selectedGPU, gpu_mask_nvidia, true);
   } else {
-    gpu_mask = UINT_MAX;
+    gpu_mask_nvidia = UINT_MAX;
   }
   if (ignoredGPU != NULL) {
-    gpu_mask = update_mask_value(ignoredGPU, gpu_mask, false);
+    gpu_mask_nvidia = update_mask_value(ignoredGPU, gpu_mask_nvidia, false);
   }
 
-  if (!init_gpu_info_extraction())
+  unsigned devices_count = 0;
+  gpu_info *devices = NULL;
+  if (!gpuinfo_init_info_extraction(gpu_mask_nvidia, &devices_count, &devices))
     return EXIT_FAILURE;
-  size_t num_devices;
-  struct device_info *dev_infos;
-  num_devices = initialize_device_info(&dev_infos, gpu_mask);
-  if (num_devices == 0) {
-    fprintf(stdout, "No GPU left to monitor.\n");
-    free(dev_infos);
+  if (devices_count == 0) {
+    fprintf(stdout, "No GPU to monitor.\n");
     return EXIT_SUCCESS;
   }
+
+  nvtop_interface_option interface_options;
+  alloc_interface_options_internals(custom_config_file_path, devices_count,
+                                    &interface_options);
+  load_interface_options_from_config_file(devices_count, &interface_options);
+  for (unsigned i = 0; i < devices_count; ++i) {
+    // Nothing specified in the file
+    if (!plot_isset_draw_info(plot_information_count,
+                              interface_options.device_information_drawn[i])) {
+      interface_options.device_information_drawn[i] = plot_default_draw_info();
+    } else {
+      interface_options.device_information_drawn[i] =
+          plot_remove_draw_info(plot_information_count,
+                                interface_options.device_information_drawn[i]);
+    }
+  }
+  if (!process_is_field_displayed(process_field_count,
+                                  interface_options.process_fields_displayed)) {
+    interface_options.process_fields_displayed =
+        process_default_displayed_field();
+  } else {
+    interface_options.process_fields_displayed =
+        process_remove_field_to_display(
+            process_field_count, interface_options.process_fields_displayed);
+  }
+  if (no_color_option)
+    interface_options.use_color = false;
+  if (hide_plot_option) {
+    for (unsigned i = 0; i < devices_count; ++i) {
+      interface_options.device_information_drawn[i] = 0;
+    }
+  }
+  if (encode_decode_timer_option_set) {
+    interface_options.encode_decode_hiding_timer = encode_decode_hide_time;
+    if (interface_options.encode_decode_hiding_timer < 0.)
+      interface_options.encode_decode_hiding_timer = 0.;
+  }
+  if (reverse_plot_direction_option)
+    interface_options.plot_left_to_right = true;
+  if (use_fahrenheit_option)
+    interface_options.temperature_in_fahrenheit = true;
+  if (update_interval_option_set)
+    interface_options.update_interval = update_interval_option;
+
+  gpuinfo_populate_static_infos(devices_count, devices);
+
   size_t biggest_name = 0;
-  for (size_t i = 0; i < num_devices; ++i) {
-    size_t device_name_size = strlen(dev_infos->device_name);
+  for (unsigned i = 0; i < devices_count; ++i) {
+    size_t device_name_size;
+    if (IS_VALID(gpuinfo_device_name_valid, devices[i].static_info.valid))
+      device_name_size = strlen(devices[i].static_info.device_name);
+    else
+      device_name_size = 4;
     if (device_name_size > biggest_name) {
       biggest_name = device_name_size;
     }
   }
-  struct nvtop_interface *interface = initialize_curses(
-      num_devices, biggest_name, use_color_if_available, use_fahrenheit,
-      show_plot, plot_old_to_recent, encode_decode_hide_time, refresh_interval);
-  timeout(refresh_interval);
+  struct nvtop_interface *interface =
+      initialize_curses(devices_count, biggest_name, interface_options);
+  timeout(interface_update_interval(interface));
 
-  double time_slept = refresh_interval;
+  double time_slept = interface_update_interval(interface);
   while (!signal_exit) {
     if (signal_resize_win) {
       update_window_size_to_terminal_size(interface);
       signal_resize_win = 0;
     }
-    if (!cache_pid_infos)
-      clean_pid_cache();
-    if (time_slept >= refresh_interval) {
-      update_device_infos(num_devices, dev_infos);
-      timeout(refresh_interval);
+    if (time_slept >= interface_update_interval(interface)) {
+      gpuinfo_refresh_dynamic_info(devices_count, devices);
+      if (!interface_freeze_processes(interface)) {
+        gpuinfo_refresh_processes(devices_count, devices);
+      }
+      save_current_data_to_ring(devices_count, devices, interface);
+      timeout(interface_update_interval(interface));
       time_slept = 0.;
     } else {
-      int next_sleep = (int)((refresh_interval - time_slept));
+      int next_sleep = interface_update_interval(interface) - (int)time_slept;
       timeout(next_sleep);
     }
-    draw_gpu_info_ncurses(dev_infos, interface);
+    draw_gpu_info_ncurses(devices_count, devices, interface);
 
     nvtop_time time_before_sleep, time_after_sleep;
     nvtop_get_current_time(&time_before_sleep);
@@ -300,16 +365,18 @@ int main(int argc, char **argv) {
     case 'q':
       signal_exit = 1;
       break;
+    case KEY_F(2):
     case KEY_F(9):
     case KEY_F(6):
+    case KEY_F(12):
     case '+':
     case '-':
       interface_key(input_char, interface);
       break;
-    case KEY_LEFT:
-    case KEY_RIGHT:
     case KEY_UP:
     case KEY_DOWN:
+    case KEY_LEFT:
+    case KEY_RIGHT:
     case KEY_ENTER:
     case '\n':
       interface_key(input_char, interface);
@@ -321,8 +388,7 @@ int main(int argc, char **argv) {
   }
 
   clean_ncurses(interface);
-  clean_device_info(num_devices, dev_infos);
-  shutdown_gpu_info_extraction();
+  gpuinfo_shutdown_info_extraction(devices_count, devices);
 
   return EXIT_SUCCESS;
 }
